      subroutine pvv_qp(ig,iwdth,u1,n1,p1,u2,n2,p2,u,n,p,g,mass,wdth)
c----------------------------------------------------------------------c
c Returns a vector array being a contraction of a triple 
c pi0-vector meson-vector meson coupling with a scalar u1 and a vector u2
c
c Input: ig=0(fixed coupling g)/else(running coupling g),
c        iwdth=0 - fixed width of the vector particle,
c              1,2,... - running width of the vector particle,
c        u1 - scalar array,
c        u2 - vector array,
c        n1,n2,n - numbers of polarization states,
c        p1,p2 - momenta,
c        g - a triple pi0-vector meson-vector meson coupling,
c        mass, wdth - mass and width of a vector particle.
c Output: u - vector array,
c         p - its momentum.
c All four momenta are cosidered as incoming to the vertex.
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 23 June, 2014; modified on 
c 12 March, 2015 (call to subroutine runwidth added)
c 19 March, 2015 (call to subroutine runwidth changed,
c                 call to subroutine runcoupl implemented)
c 9 March, 2020 (quadruple precision version written)
c 25 Feb. 2021: QP version modified
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0), qp=kind(1.q0)
      integer :: ig,iwdth,n1,n2,n,i1,i2,i

      real(dp) :: mass,wdth
      real(qp), dimension(0:3) :: p1,p2,p
      complex(dp) :: g,rg,mv2
      complex(qp) :: u1(1,n1),u2(0:3,n2),u(0:3,n),gden

      p=-p1-p2

      if (ig == 0) then
       rg=g
      else
       call runcoupl_qp(ig,p,rg)
      end if

      if (mass > 0.d0) then
       if (iwdth == 0) then
        mv2=cmplx(mass**2,-mass*wdth,dp)
       else
        call runwidth_qp(iwdth,p,mass,wdth,mv2)
       end if
       gden=-rg/(cmplx(p(0)**2-p(1)**2-p(2)**2-p(3)**2,0.q0,qp)-mv2)
      else
       gden=-rg/cmplx(p(0)**2-p(1)**2-p(2)**2-p(3)**2,0.q0,qp)
      end if

      i=0
      do i2=1,n2
       do i1=1,n1
        i=i+1
        u(0,i)=gden*u1(1,i1)*(
     &             -u2(1,i2)*cmplx(p2(2)*p(3)-p2(3)*p(2),0.q0,qp)
     &             +u2(2,i2)*cmplx(p2(1)*p(3)-p2(3)*p(1),0.q0,qp)
     &             -u2(3,i2)*cmplx(p2(1)*p(2)-p2(2)*p(1),0.q0,qp))
        u(1,i)=gden*u1(1,i1)*(
     &             -u2(0,i2)*cmplx(p2(2)*p(3)-p2(3)*p(2),0.q0,qp)
     &             +u2(2,i2)*cmplx(p2(0)*p(3)-p2(3)*p(0),0.q0,qp)
     &             -u2(3,i2)*cmplx(p2(0)*p(2)-p2(2)*p(0),0.q0,qp))
        u(2,i)=gden*u1(1,i1)*(
     &              u2(0,i2)*cmplx(p2(1)*p(3)-p2(3)*p(1),0.q0,qp)
     &             -u2(1,i2)*cmplx(p2(0)*p(3)-p2(3)*p(0),0.q0,qp)
     &             +u2(3,i2)*cmplx(p2(0)*p(1)-p2(1)*p(0),0.q0,qp))
        u(3,i)=gden*u1(1,i1)*(
     &             -u2(0,i2)*cmplx(p2(1)*p(2)-p2(2)*p(1),0.q0,qp)
     &             +u2(1,i2)*cmplx(p2(0)*p(2)-p2(2)*p(0),0.q0,qp)
     &             -u2(2,i2)*cmplx(p2(0)*p(1)-p2(1)*p(0),0.q0,qp))
        if (mass > 0.d0) then
         u(:,i)=u(:,i)-(p(0)*u(0,i)-p(1)*u(1,i)
     &                 -p(2)*u(2,i)-p(3)*u(3,i))*p/mv2
        end if
       end do
      end do

      end subroutine pvv_qp
