      subroutine svsvkk(ig,iwdth,u1,n1,p1,u2,n2,p2,u3,n3,p3,u,n,p,
     &                                                     g,mass,wdth)
c----------------------------------------------------------------------c
c Returns a particle being a contraction of a quartic gauge-scalar boson 
c vertex with two polarization vectors and one scalar.
c
c Input: ig=0(fixed coupling g)/else(running coupling g),
c        iwdth=0 - fixed width of the vector particle,
c              1,2,... - running width of the vector particle,
c        u2 - vector array,
c        u1,u3 - scalar arrays,
c        n1,n2,n3,n - numbers of polarization states,
c        p1,p2,p3 - momenta,
c        g - a quartic Higgs-gauge-boson coupling,
c        mass, wdth - mass and width of u.
c Output: u - vector array,
c         p - its momentum.
c All four momenta are cosidered as incoming to the vertex.
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 3 June, 2007 (based on vssvkk.f); modified on 
c 1 April, 2015:
c calls to subroutine runwidth and runcoupl implemented
c 6 April, 2015:
c an extra minus sign, resulting from missing i^2 if a quartic coupling 
c is present, introduced
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0)
      integer :: ig,iwdth,n1,n2,n3,n,i1,i2,i3,i

      real(dp) :: mass,wdth
      real(dp), dimension(0:3) :: p1,p2,p3,p
      complex(dp) :: u2(0:3,n1),u(0:3,n2),u1(1,n3),u3(1,n),g,rg,gden,mv2

      p=-p1-p2-p3

      if (ig == 0) then
       rg=g
      else
       call runcoupl(ig,p,rg)
      end if

      if (mass > 0.d0) then
       if (iwdth == 0) then
        mv2=cmplx(mass**2,-mass*wdth,dp)
       else
        call runwidth(iwdth,p,mass,wdth,mv2)
       end if
       gden=rg/(cmplx(p(0)**2-p(1)**2-p(2)**2-p(3)**2,0.d0,dp)-mv2)
      else
       gden=rg/cmplx(p(0)**2-p(1)**2-p(2)**2-p(3)**2,0.d0,dp)
      end if

      i=0

      do i3=1,n3
       do i2=1,n2
        do i1=1,n1
         i=i+1
         u(:,i)=gden*u1(1,i2)*u3(1,i3)*u2(:,i1)
         if (mass > 0.d0) then
          u(:,i)=u(:,i)
     &             -(p(0)*u(0,i)-p(1)*u(1,i)
     &              -p(2)*u(2,i)-p(3)*u(3,i))*p/mv2
         end if
        end do
       end do
      end do

      end subroutine svsvkk
