      subroutine vff_qp(u1,n1,p1,u2,n2,p2,u,n,p,ip,g,mass,wdth)
c----------------------------------------------------------------------c
c Returns the spinor array
c                  u=f*e*(gp*P+ + gm*P-)*(p+m)/den
c
c Input: u1 - polarization vector,
c        u2 - spinor
c        p1,p2 - momenta,
c        n1,n2,n - numbers of polarization states,
c        g - a fermion-fermion-vector boson coupling,
c        mass, wdth - a mass width of u
c        ip=+1/-1 for u representing a particle/an antiparticle
c All four momenta are cosidered as incoming to the vertex.
c
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 16 May, 2007 (name and interface changed, used to be vfkk); 
c                           modified on 
c 8 Nov. 2018: quadrple precision four momenta implemented
c 25 Feb. 2021: QP version modified
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0),qp=kind(1.q0)
      complex(dp), parameter :: ocom=(0.d0,0.d0)
      complex(qp), parameter :: icom=(0.q0,1.q0)
      integer :: n1,n2,n,ip,i1,i2,i
      real(qp), dimension(0:3) :: p1,p2,p
      real(dp) :: mass,wdth
      complex(dp) :: g(2)
      complex(qp) :: mf2,den,u1(0:3,n1),u2(4,n2),u(4,n),
     &               mf,gp,gm,mgp,mgm,epu(2),emu(2)
      complex(qp), dimension(4) :: pp,pm

      mf2=cmplx(mass**2,-mass*wdth,qp)
      mf=sqrt(mf2)

      p=p1+p2

      pp=ip*(/cmplx(p(0)-p(3),0.q0,qp),   ! (1,1)
     &        cmplx(-p(1),-p(2),qp),      ! (2,1)
     &        cmplx(-p(1),p(2),qp),       ! (1,2)
     &        cmplx(p(0)+p(3),0.q0,qp)/)  ! (2,2)

      pm=(/pp(4),-pp(2),-pp(3),pp(1)/)

      den=cmplx((p(0)-p(3))*(p(0)+p(3))-p(1)**2-p(2)**2,0.q0,qp)-mf2
c      den=pp(1)*pp(4)-pp(3)*pp(2)-mf2

      gp=g(1)/den
      gm=g(2)/den
      mgp=mf*gp
      mgm=mf*gm

      i=0

      do i2=1,n2
       do i1=1,n1
        i=i+1
        emu= (/(u1(0,i1)+u1(3,i1))*u2(1,i2)
     &        +(u1(1,i1)-icom*u1(2,i1))*u2(2,i2),
     &         (u1(1,i1)+icom*u1(2,i1))*u2(1,i2)
     &        +(u1(0,i1)-u1(3,i1))*u2(2,i2)/)
        u(:,i)=(/gm*(pp(1)*emu(1)+pp(3)*emu(2)),
     &           gm*(pp(2)*emu(1)+pp(4)*emu(2)),
     &           mgm*emu(1),
     &           mgm*emu(2)/)
        if (g(1) /= ocom) then
         epu= (/(u1(0,i1)-u1(3,i1))*u2(3,i2)
     &        +(-u1(1,i1)+icom*u1(2,i1))*u2(4,i2),
     &         (-u1(1,i1)-icom*u1(2,i1))*u2(3,i2)
     &         +(u1(0,i1)+u1(3,i1))*u2(4,i2)/)
         u(:,i)=u(:,i)+(/mgp*epu(1),
     &                   mgp*epu(2),
     &                   gp*(pm(1)*epu(1)+pm(3)*epu(2)),
     &                   gp*(pm(2)*epu(1)+pm(4)*epu(2))/)
        end if
       end do
      end do

      end subroutine vff_qp
