      subroutine annmd(ip,mu,pe,e0,e,bu,u,mat)
c----------------------------------------------------------------------c
c Returns the matrix element bu*e_n*Gamma^n*u with
c    Gamma^n=e0*[F_1(q^2)*gamma^n+F_2(q^2)*i*sigma^nm*q_m]
c representing an effective nucleon-nucleon-photon coupling. 
c Factor 1/(2m) included in F_2. Based on a routine wbtmd.f.
c
c Input: 
c        ip=+1/-1 for u representing a particle/an antiparticle,
c        mu - mass of a nucleon reprezented by u and ub,
c        pe - four momentum of incoming e,
c        e0 - electric charge,
c        ub,u - nucleon spinors,
c        e -  polarization vector of a photon.
c
c The Dirac gamma matrices are in the Weyl representation
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 19 Feb./13 March, 2014
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0)

      integer :: ip
      real(dp), dimension(0:3) :: pe
      complex(dp) :: mat
      complex(dp), dimension(0:3) :: e
      complex(dp), dimension(4) :: bu,u
      real(dp) :: mu,e0
      complex(dp) :: f1,f2

c Local variables:

      complex(dp), parameter :: icom=(0.d0,1.d0)
      complex(dp) :: f1l,f1r,f2l,f2r
      complex(dp), dimension(4) :: u1
      complex(dp), dimension(2,2) :: b,c,ae,de
      complex(dp), dimension(0:3,2,2) :: a,d
      real(dp) :: q2

      q2=pe(0)**2-pe(1)**2-pe(2)**2-pe(3)**2

      call nuclff_phok(mu,q2,f1,f2)

      f1r=e0*f1
      f1l=f1r
      f2r=e0*f2
      f2l=f2r

      if (ip == -1) then
       f1r=conjg(f1r)
       f1l=conjg(f1l)
       f2r=conjg(f2r)
       f2l=conjg(f2l)
      end if

      a(0,1,:)=(/cmplx(pe(3),0.d0,dp),cmplx(pe(1),-pe(2),dp)/)
      a(0,2,:)=(/cmplx(pe(1),pe(2),dp),-a(0,1,1)/)
      a(1,1,:)=(/cmplx(0.d0,pe(2),dp),cmplx(pe(0)-pe(3),0.d0,dp)/)
      a(1,2,:)=(/cmplx(pe(0)+pe(3),0.d0,dp),-a(1,1,1)/)
      a(2,1,:)=(/cmplx(0.d0,-pe(1),dp),cmplx(0.d0,-pe(0)+pe(3),dp)/)
      a(2,2,:)=(/cmplx(0.d0,pe(0)+pe(3),dp),-a(2,1,1)/)
      a(3,1,:)=(/cmplx(pe(0),0.d0,dp),a(0,1,2)/)
      a(3,2,:)=(/-a(0,2,1),-a(3,1,1)/)
      d(0,:,:)=-a(0,:,:)
      d(1,1,:)=(/a(1,1,1),-a(1,2,1)/)
      d(1,2,:)=(/-a(1,1,2),-a(1,1,1)/)
      d(2,1,:)=(/a(2,1,1),a(2,2,1)/)
      d(2,2,:)=(/a(2,1,2),a(2,2,2)/)
      d(3,1,:)=(/a(3,2,2),a(3,1,2)/)
      d(3,2,:)=(/a(3,2,1),a(3,1,1)/)

      b(1,:)=(/e(0)-e(3),-e(1)+icom*e(2)/)
      b(2,:)=(/-e(1)-icom*e(2),e(0)+e(3)/)
      c(1,:)=(/b(2,2),-b(1,2)/)
      c(2,:)=(/-b(2,1),b(1,1)/)
      ae=a(0,:,:)*e(0)-a(1,:,:)*e(1)-a(2,:,:)*e(2)-a(3,:,:)*e(3)
      de=d(0,:,:)*e(0)-d(1,:,:)*e(1)-d(2,:,:)*e(2)-d(3,:,:)*e(3)
      u1(:2)=f2l*matmul(ae,bu(:2))+f1r*matmul(b,bu(3:))
      u1(3:)=f1l*matmul(c,bu(:2))+f2r*matmul(de,bu(3:))

      mat=u1(1)*u(1)+u1(2)*u(2)+u1(3)*u(3)+u1(4)*u(4)

      end




