      subroutine btwmd_qp(q,gwtb,ub,u,e,mat)
c----------------------------------------------------------------------c
c Returns the matrix element bu*e*gwtb*u with the anomalous Wtb coupling 
c
c Input: 
c        q - four momentum of incoming e,
c        gwtb - the coupling given by Eq. (4) of K. Kolodziej, Phys. 
c               Lett. B584 (2004) 89, hep-ph/0312168,
c        ub - spinor of the b-quark,
c        u - spinor of the t-quark,
c        e -  polarization vector of W
c
c The Dirac gamma matrices are in the Weyl representation
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 10 Feb., 2011
c 11 April, 2011: error in a(2,2,1) corrected
c 6 March, 2020: 
c quadruple precision version written
c 25 Feb. 2021: QP version modified
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0), qp=kind(1.q0)

      real(qp), dimension(0:3) :: q,pe
      complex(dp) :: gwtb(2,4)
      complex(qp) :: mat
      complex(qp), dimension(4) :: ub,u
      complex(qp), dimension(0:3) :: e

c Local variables:

      integer :: l
      complex(qp), parameter :: icom=(0.q0,1.q0)
      complex(dp) :: f1l,f1r,f2l,f2r
      complex(qp), dimension(0:3,2,2) :: a,d
      complex(qp), dimension(0:3) :: ew

      f1r=gwtb(1,1)
      f1l=gwtb(2,1)
      f2r=gwtb(1,2)
      f2l=gwtb(2,2)

      pe=-q ! Vertex is parametrized in terms of outgoing W four momentum

      a(0,1,:)=(/cmplx(pe(3),0.q0,qp),cmplx(pe(1),-pe(2),qp)/)
      a(0,2,:)=(/cmplx(pe(1),pe(2),qp),-a(0,1,1)/)
      a(1,1,:)=(/cmplx(0.q0,pe(2),qp),cmplx(pe(0)-pe(3),0.q0,qp)/)
      a(1,2,:)=(/cmplx(pe(0)+pe(3),0.q0,qp),-a(1,1,1)/)
      a(2,1,:)=(/cmplx(0.q0,-pe(1),qp),cmplx(0.q0,-pe(0)+pe(3),qp)/)
      a(2,2,:)=(/cmplx(0.q0,pe(0)+pe(3),qp),-a(2,1,1)/)
      a(3,1,:)=(/cmplx(pe(0),0.q0,qp),a(0,1,2)/)
      a(3,2,:)=(/-a(0,2,1),-a(3,1,1)/)
      d(0,:,:)=-a(0,:,:)
      d(1,1,:)=(/a(1,1,1),-a(1,2,1)/)
      d(1,2,:)=(/-a(1,1,2),-a(1,1,1)/)
      d(2,1,:)=(/a(2,1,1),a(2,2,1)/)
      d(2,2,:)=(/a(2,1,2),a(2,2,2)/)
      d(3,1,:)=(/a(3,2,2),a(3,1,2)/)
      d(3,2,:)=(/a(3,2,1),a(3,1,1)/)

      do l=0,3
       ew(l)=f2l*(ub(1)*(a(l,1,1)*u(1)+a(l,1,2)*u(2))
     &           +ub(2)*(a(l,2,1)*u(1)+a(l,2,2)*u(2)))
     &      +f2r*(ub(3)*(d(l,1,1)*u(3)+d(l,1,2)*u(4))
     &           +ub(4)*(d(l,2,1)*u(3)+d(l,2,2)*u(4)))
      end do
      ew(0)=ew(0)+f1l*(ub(3)*u(1)+ub(4)*u(2))
     &           +f1r*(ub(1)*u(3)+ub(2)*u(4))
      ew(1)=ew(1)-f1l*(ub(3)*u(2)+ub(4)*u(1))
     &           +f1r*(ub(1)*u(4)+ub(2)*u(3))
      ew(2)=ew(2)-f1l*(-ub(3)*u(2)+ub(4)*u(1))*icom
     &           +f1r*(-ub(1)*u(4)+ub(2)*u(3))*icom
      ew(3)=ew(3)-f1l*(ub(3)*u(1)-ub(4)*u(2))
     &           +f1r*(ub(1)*u(3)-ub(2)*u(4))

      mat=ew(0)*e(0)-ew(1)*e(1)-ew(2)*e(2)-ew(3)*e(3)

      end




