      subroutine ffvmd_qp(g,u1,u2,u,mat)
c----------------------------------------------------------------------c
c Returns a matrix element
c           u=u1*gamma_n*(gvm*P- + gvp*P+)*u2.u
c Input: g - a fermion-fermion-vector boson coupling.
c        u1,u2 - spinors,
c        u - vector.
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 15 May, 2007; modified 23 May, 2007
c 23 Feb. 2021: QP version
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0),qp=kind(1.q0)

      complex(dp), parameter :: ocom=(0.d0,0.d0)
      complex(qp), parameter :: icom=(0.q0,1.q0)
      complex(dp) :: g(2)
      complex(qp) :: u1(4),u2(4),u(0:3),mat
      complex(qp),dimension(0:3) :: em,ep

       em=g(2)*(/ u1(3)*u2(1)+u1(4)*u2(2),
     &          -(u1(3)*u2(2)+u1(4)*u2(1)),
     &     +icom*(u1(3)*u2(2)-u1(4)*u2(1)),
     &           -u1(3)*u2(1)+u1(4)*u2(2)/)

      if (g(1) == ocom) then

       ep=ocom

      else

       ep=g(1)*(/u1(1)*u2(3)+u1(2)*u2(4),
     &           u1(1)*u2(4)+u1(2)*u2(3),
     &    -icom*(u1(1)*u2(4)-u1(2)*u2(3)),
     &           u1(1)*u2(3)-u1(2)*u2(4)/)

      end if

      mat=(em(0)+ep(0))*u(0)-(em(1)+ep(1))*u(1)-(em(2)+ep(2))*u(2)
     &                                         -(em(3)+ep(3))*u(3)

      end subroutine ffvmd_qp
