      subroutine ppamd_qp(ig,sgn,g,u1,p1,u2,p2,u3,mat)
c----------------------------------------------------------------------c
c Returns a matrix element array for the a triple pi+pi-photon 
c vertex of the scalar QED
c
c Input: ig=0(fixed coupling g)/else(running coupling g),
c        sgn=+1, if scalars are writen in order pi-pi+,
c            -1, if scalars are writen in order pi+pi-,
c        u1,u2 - scalars,
c        u3 - vector
c        g - a triple pi+pi-photon coupling,
c All four momenta are cosidered as incoming to the vertex.
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 30 April, 2009; modified on 
c 19 March, 2015 (call to subroutine runcoupl implemented)
c 3 March, 2020 (quadruple precision version written)
c 25 Feb. 2021: QP version modified
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0), qp=kind(1.q0)
      integer :: ig,sgn
      real(qp), dimension(0:3) :: p1,p2
      complex(dp) :: g,rg
      complex(qp) :: u1,u2,u3(0:3),mat

      if (ig == 0) then
       rg=g
      else
       call runcoupl_qp(ig,-p1-p2,rg)
      end if

      mat=sgn*rg*u1*u2*(u3(0)*(p1(0)-p2(0))-u3(1)*(p1(1)-p2(1))
     &                 -u3(2)*(p1(2)-p2(2))-u3(3)*(p1(3)-p2(3)))

      end subroutine ppamd_qp
