      subroutine vvv_qp(iv,u1,n1,p1,u2,n2,p2,u,n,p,g,mass,wdth)
c----------------------------------------------------------------------c
c Returns a polarization vector array being a contraction of
c a triple gauge boson vertex with two polarization vectors
c
c Input: 
c     iv = 1, for u1 being 1st and u2 the 2nd particle in the vertex,
c     iv = 2, for u1 being 1st and u2 the 3rd particle in the vertex,
c     iv = 3, for u1 being 2nd and u2 the 1st particle in the vertex,
c     iv = 4, for u1 being 2nd and u2 the 3rd particle in the vertex,
c     iv = 5, for u1 being 3rd and u2 the 1st particle in the vertex,
c     iv = 6, for u1 being 3rd and u2 the 2nd particle in the vertex,
c     p1,p2 - momenta,
c     u1,u2 - polarization vectors,
c     n1,n2,n - numbers of polarization states,
c     g - a triple-boson coupling,
c     mass, wdth - mass and width of u.
c Output:
c     u - polarization vector,
c     p - momentum of u.
c All four momenta are cosidered as incoming to the vertex.
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 27 March, 2006; modified on 16 May, 2007 (interface changed),
c                             21 May, 2007 (bug introduced on 16 May
c                                           fixed)
c 9 March, 2020 quadruple precision version written
c 25 Feb. 2021: QP version modified
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0), qp=kind(1.q0)
      integer :: iv,n1,n2,n,i1,i2,i

      real(qp), dimension(0:3) :: p1,p2,p
      real(dp) :: mass,wdth
      complex(dp) :: g,mv2
      complex(qp) :: u1(0:3,n1),u2(0:3,n2),u(0:3,n),gden

      p=-p1-p2

      mv2=cmplx(mass**2,-mass*wdth,dp)
      gden=g/(cmplx(p(0)**2-p(1)**2-p(2)**2-p(3)**2,0.q0,qp)-mv2)

      i=0

      if (iv == 1) then
       do i2=1,n2
        do i1=1,n1
         i=i+1
         u(:,i)=-gden*((u1(0,i1)*(p2(0)-p(0))
     &              -u1(1,i1)*(p2(1)-p(1))
     &              -u1(2,i1)*(p2(2)-p(2))
     &              -u1(3,i1)*(p2(3)-p(3)))*u2(:,i2)
     &             +(u2(0,i2)*(p(0)-p1(0))
     &              -u2(1,i2)*(p(1)-p1(1))
     &              -u2(2,i2)*(p(2)-p1(2))
     &              -u2(3,i2)*(p(3)-p1(3)))*u1(:,i1)
     &             +(u1(0,i1)*u2(0,i2)
     &              -u1(1,i1)*u2(1,i2)
     &              -u1(2,i1)*u2(2,i2)
     &              -u1(3,i1)*u2(3,i2))*(p1-p2))
         if (mass /= 0.d0) then
          u(:,i)=u(:,i)-(p(0)*u(0,i)-p(1)*u(1,i)
     &                  -p(2)*u(2,i)-p(3)*u(3,i))*p/mv2
         end if
        end do
       end do
      else if (iv == 2) then
       do i2=1,n2
        do i1=1,n1
         i=i+1
         u(:,i)=-gden*((u1(0,i1)*(p(0)-p2(0))
     &              -u1(1,i1)*(p(1)-p2(1))
     &              -u1(2,i1)*(p(2)-p2(2))
     &              -u1(3,i1)*(p(3)-p2(3)))*u2(:,i2)
     &             +(u2(0,i2)*(p1(0)-p(0))
     &              -u2(1,i2)*(p1(1)-p(1))
     &              -u2(2,i2)*(p1(2)-p(2))
     &              -u2(3,i2)*(p1(3)-p(3)))*u1(:,i1)
     &             +(u1(0,i1)*u2(0,i2)
     &              -u1(1,i1)*u2(1,i2)
     &              -u1(2,i1)*u2(2,i2)
     &              -u1(3,i1)*u2(3,i2))*(p2-p1))
         if (mass /= 0.d0) then
          u(:,i)=u(:,i)-(p(0)*u(0,i)-p(1)*u(1,i)
     &                  -p(2)*u(2,i)-p(3)*u(3,i))*p/mv2
         end if
        end do
       end do
      else if (iv == 3) then
       do i2=1,n2
        do i1=1,n1
         i=i+1
         u(:,i)=-gden*((u2(0,i2)*(p1(0)-p(0))
     &              -u2(1,i2)*(p1(1)-p(1))
     &              -u2(2,i2)*(p1(2)-p(2))
     &              -u2(3,i2)*(p1(3)-p(3)))*u1(:,i1)
     &             +(u1(0,i1)*(p(0)-p2(0))
     &              -u1(1,i1)*(p(1)-p2(1))
     &              -u1(2,i1)*(p(2)-p2(2))
     &              -u1(3,i1)*(p(3)-p2(3)))*u2(:,i2)
     &             +(u2(0,i2)*u1(0,i1)
     &              -u2(1,i2)*u1(1,i1)
     &              -u2(2,i2)*u1(2,i1)
     &              -u2(3,i2)*u1(3,i1))*(p2-p1))
         if (mass /= 0.d0) then
          u(:,i)=u(:,i)-(p(0)*u(0,i)-p(1)*u(1,i)
     &                  -p(2)*u(2,i)-p(3)*u(3,i))*p/mv2
         end if
        end do
       end do
      else if (iv == 4) then
       do i2=1,n2
        do i1=1,n1
         i=i+1
         u(:,i)=-gden*((u2(0,i2)*(p(0)-p1(0))
     &              -u2(1,i2)*(p(1)-p1(1))
     &              -u2(2,i2)*(p(2)-p1(2))
     &              -u2(3,i2)*(p(3)-p1(3)))*u1(:,i1)
     &             +(u1(0,i1)*(p2(0)-p(0))
     &              -u1(1,i1)*(p2(1)-p(1))
     &              -u1(2,i1)*(p2(2)-p(2))
     &              -u1(3,i1)*(p2(3)-p(3)))*u2(:,i2)
     &             +(u1(0,i1)*u2(0,i2)
     &              -u1(1,i1)*u2(1,i2)
     &              -u1(2,i1)*u2(2,i2)
     &              -u1(3,i1)*u2(3,i2))*(p1-p2))
         if (mass /= 0.d0) then
          u(:,i)=u(:,i)-(p(0)*u(0,i)-p(1)*u(1,i)
     &                  -p(2)*u(2,i)-p(3)*u(3,i))*p/mv2
         end if
        end do
       end do
      else if (iv == 5) then
       do i2=1,n2
        do i1=1,n1
         i=i+1
         u(:,i)=-gden*((u2(0,i2)*(p(0)-p1(0))
     &              -u2(1,i2)*(p(1)-p1(1))
     &              -u2(2,i2)*(p(2)-p1(2))
     &              -u2(3,i2)*(p(3)-p1(3)))*u1(:,i1)
     &             +(u1(0,i1)*(p2(0)-p(0))
     &              -u1(1,i1)*(p2(1)-p(1))
     &              -u1(2,i1)*(p2(2)-p(2))
     &              -u1(3,i1)*(p2(3)-p(3)))*u2(:,i2)
     &             +(u1(0,i1)*u2(0,i2)
     &              -u1(1,i1)*u2(1,i2)
     &              -u1(2,i1)*u2(2,i2)
     &              -u1(3,i1)*u2(3,i2))*(p1-p2))
         if (mass /= 0.d0) then
          u(:,i)=u(:,i)-(p(0)*u(0,i)-p(1)*u(1,i)
     &                  -p(2)*u(2,i)-p(3)*u(3,i))*p/mv2
         end if
        end do
       end do
      else if (iv == 6) then
       do i2=1,n2
        do i1=1,n1
         i=i+1
         u(:,i)=-gden*((u2(0,i2)*(p1(0)-p(0))
     &              -u2(1,i2)*(p1(1)-p(1))
     &              -u2(2,i2)*(p1(2)-p(2))
     &              -u2(3,i2)*(p1(3)-p(3)))*u1(:,i1)
     &             +(u1(0,i1)*(p(0)-p2(0))
     &              -u1(1,i1)*(p(1)-p2(1))
     &              -u1(2,i1)*(p(2)-p2(2))
     &              -u1(3,i1)*(p(3)-p2(3)))*u2(:,i2)
     &             +(u1(0,i1)*u2(0,i2)
     &              -u1(1,i1)*u2(1,i2)
     &              -u1(2,i1)*u2(2,i2)
     &              -u1(3,i1)*u2(3,i2))*(p2-p1))
         if (mass /= 0.d0) then
          u(:,i)=u(:,i)-(p(0)*u(0,i)-p(1)*u(1,i)
     &                  -p(2)*u(2,i)-p(3)*u(3,i))*p/mv2
         end if
        end do
       end do
      else
       write(*,*)'vvvkk.f: No such option, iv=',iv
      end if

      end subroutine vvv_qp
