      function crosskk(x,ndim)
c----------------------------------------------------------------------c
c The unpolarized cross section
c
c Input:  x(ndim) - random numbers supplied by carlos.
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 23 March, 2006, modiffied on 15 Nov. 2007;
c                              29 Oct. 2008;
c                              31 Jul. 2009
c                              7 Aug. 2009 (ISR implemented)
c 13 March 2012: a call to Ctq6pdf corrected, scale qsc changed
c 30 March, 2016: a condition if dlinv <= 0 then crosskk=0 added
c 16/19 Sept. 2022: calls to kincls modified, such that there are the
c same for both iqprec=0 and iqprec=1
c----------------------------------------------------------------------c
      use inprms
      use drivekk
      use distribs
      use kincuts
      use prtinds
      use psnrmtest
      use ntdisc
      use fourmom

      implicit none

      integer :: ndim,i,ifrst=1,istf,imxup=0

      real(dp) :: crosskk,x(ndim),ecm0=zero,flux,dlinv,dlips,
     &            mat2,cps,phi(nmcs),p_dp(0:3,nepl)
      real(qp) :: ecm_qp

      real :: y(nmcs),rand

      real(dp), parameter :: c=2.d0/(pi*ialph0)
      real(dp) :: ecmp,gllee,x1,x2,
     &            L1,beta1,eta1,betinv1,betahf1,
     &            L2,beta2,eta2,betinv2,betahf2,
     &            dx1,dx2,y1,y2,r(0:3,nepl),!vz,gz,
     &            qsc,Ctq6pdf,logx0,xpdf1,xpdf2,ypdf1,ypdf2,GetOnePDF,
     &            q2had,dq2had

      character(80) ind

      save ifrst,cps,ecm0,flux,imxup,logx0

      if (ifrst == 1) then
       cps=(twopi)**(4-3*(nepl-nisp))
       if (ihad > 0) then
        logx0=log(x0pdf)
       end if
       ifrst=0
      end if

      if (iisr == 1) then
       L1=log(qsc2/m(1)**2)
       L2=log(qsc2/m(2)**2)
       eta1=c*L1
       eta2=c*L2
       beta1=c*(L1-1.d0)
       beta2=c*(L2-1.d0)
       betinv1=2.d0/beta1
       betinv2=2.d0/beta2
       betahf1=1.d0/betinv1
       betahf2=1.d0/betinv2
       x1=x(ndim-1)
       x2=x(ndim)

       y1=x1**betinv1
       x1=1.d0-y1

       if (y1 < epsisr) then
        dx1=betinv1
       else
        dx1=betinv1*y1**(1.d0-betahf1)
       end if

       y2=x2**betinv2
       x2=1.d0-y2

       if (y2 < epsisr) then
        dx2=betinv2
       else
        dx2=betinv2*y2**(1.d0-betahf2)
       end if

c       ecmp=sqrt(x1*x2*ecm2+((y2-y1)*me)**2)
       ecmp=sqrt(x1*x2)*ecm
c       ecmp=ecm
       if (ecmp < mthrs) then
        crosskk=0.d0
        return
       end if
       if (iqprec == 1) then
        ecm_qp=ecmp
        call kinin_qp(ecm_qp,m(1:2),p(:,1:2),flux)
       else
        call kinin(ecmp,m(1:2),p(:,1:2),flux)
       end if
      else if (iisr == 2) then
       q2had=ecm2-(ecm2-q2_min)
     &      *((ecm2-q2_max)/(ecm2-q2_min))**x(ndim)
       dq2had=-(ecm2-q2had)*log((ecm2-q2_max)/(ecm2-q2_min))
c       dq2had=q2_max-q2_min
c       q2had=dq2had*x(ndim)+q2_min
       ecmp=sqrt(q2had)
       if (iqprec == 1) then
        ecm_qp=ecmp
        call kinin_qp(ecm_qp,m(1:2),p(:,1:2),flux)
       else
        call kinin(ecmp,m(1:2),p(:,1:2),flux)
       end if
      else if (ihad > 0) then
       x1=x0pdf**(1.d0-x(ndim-1))
       x2=x0pdf**(1.d0-x(ndim))
       dx1=-x1*logx0
       dx2=-x2*logx0
       ecmp=sqrt(x1*x2)*ecm
       if (ecmp < mthrs) then
        crosskk=0.d0
        return
       else
        if (iscl == 1) then
         qsc=sclf*sqrt(ecmp)
        else if (iscl == 2) then
         qsc=sclf
        else if (iscl /= 3) then
         write(7,*) 'crosskk.f: This scale choice is not possible'
         stop
        end if
       end if
       if (iqprec == 1) then
        ecm_qp=ecmp
        call kinin_qp(ecm_qp,m(1:2),p(:,1:2),flux)
       else
        call kinin(ecmp,m(1:2),p(:,1:2),flux)
       end if
      else
       if (ecm /= ecm0) then
        if (iqprec == 1) then
         ecm_qp=ecm
         call kinin_qp(ecm_qp,m(1:2),p(:,1:2),flux)
        else
         call kinin(ecm,m(1:2),p(:,1:2),flux)
        end if
        ecm0=ecm
       end if
      end if ! iisr

      if (iisr == 1 .or. ihad > 0) then
       call kincls(ikin,1,x(:ndim-2),ndim-2,dlinv)
      else if (iisr == 2) then
       call kincls(ikin,1,x(:ndim-1),ndim-1,dlinv)
      else
       call kincls(ikin,1,x,ndim,dlinv)
      end if

      if (dlinv > 0.d0) then ! Introduced on 15 Nov. 2007

       if ((iisr == 1 .or. ihad > 0) .and. icuts == 1) then
c Boost momenta along the z axis back to the lab system
        if (iisr == 1) then
         vz=(y1-y2)/(x1+x2)
        else if (ihad > 0) then
         vz=(x1-x2)/(x1+x2)
        end if
        gz=0.5d0*(x1+x2)/sqrt(x1*x2)
        do i=1,nepl
         if (iqprec == 1) then
          r_qp(:,i)=(/gz*(p(0,i)+vz*p(3,i)),p(1,i),p(2,i),
     &                          gz*(p(3,i)+vz*p(0,i))/)
         else
          r(:,i)=(/gz*(p(0,i)+vz*p(3,i)),p(1,i),p(2,i),
     &                          gz*(p(3,i)+vz*p(0,i))/)
         end if
        end do
       end if

c The following relationships must hold:
c       write(*,*)'elab:',(x1+x2)*ecm/2.d0,r(0,1)+r(0,2)
c       write(*,*)'plab:',(x1-x2)*ecm/2.d0,r(3,1)+r(3,2)

       if (icuts == 1) then
        if (iisr == 1 .or. ihad > 0) then
         if (iqprec == 1) then
          r=r_qp
         end if
         call impcuts(r,istf)
        else
         p_dp=p
         call impcuts(p_dp,istf)
        end if
        if (istf == 0) then
         crosskk=0.d0
         return
        end if
        if (imtth == 1) then
         p_dp=p
         call invmcuts(p_dp,istf)
         if (istf == 0) then
          crosskk=0.d0
          return
         end if
        end if
       end if ! icuts

       if (iisr == 1 .or. ihad > 0) then
        call psncls(ikin,x(:ndim-2),ndim-2,dlinv)
       else if (iisr == 2) then
        call psncls(ikin,x(:ndim-1),ndim-1,dlinv)
       else
        call psncls(ikin,x,ndim,dlinv)
       end if

       if (dlinv > 0.d0) then    ! 30 March, 2016
        dlips=cps/dlinv
       else                      !> 30 March, 2016
        crosskk=0.d0
        return
       end if                    !< 30 March, 2016

c MC summing over polarizations:

       if (nmcs > 0) then
        if (irand == 1) then
         call random_number(y)
        else if (irand == 2) then
         call ranlux(y,nmcs)
        end if
        do i=1,nmcs
         phi(i)=twopi*y(i)
        end do
       end if

       if (iqprec == 1) then
        if (ips_vol == 1) then
         mat2=1.d0
        else
         call matrixel(phi,mat2)
        end if
       else
        if (ips_vol == 1) then
         mat2=1.d0
        else
         call matrixel(phi,mat2)
        end if
       end if

       crosskk=convc/flux*ralpw0*mat2*dlips

       if (ipsnrm == 1) then
        crosskk=crosskk/dlips*(cps/dlinvt)
       end if

       if (ipiff == 1) then
        crosskk=crosskk/(convc*pi/(3.d0*ialph0**2*ecm2)
     &                 *(sqrt(1.d0-(2.d0*mpi/ecm)**2))**3)
       end if

       if (iisr == 1) then
        crosskk=gllee(x1,y1,epsisr,qsc2,beta1,eta1,chce)*dx1
     &        *gllee(x2,y2,epsisr,qsc2,beta2,eta2,chce)*dx2*crosskk
       else if (iisr == 2) then
c        crosskk=sig_fct*crosskk/fct_pt
        crosskk=crosskk/(ialph0*pi*ecm2)
     &   *((ecm2**2+q2had**2)/(ecm2*(ecm2-q2had))*lctmin
     &     -(ecm2-q2had)/ecm2*cth_min)*dq2had
ckk_had>
       else if (ihad > 0) then
        if (iscl == 3) then
         qsc=mt**2
         do i=nisp+1,nepl
          if (iyj(i) == i .or. iyb(i) == i) then
           qsc=qsc+p(1,i)**2+p(2,i)**2
          end if
         end do
         qsc=sclf*sqrt(qsc)
        end if

        if (ihad == 1) then ! proton-anti-proton scattering
         if (ipdf == 1) then
          if (ipart(1) < 0) then
           xpdf1=GetOnePDF(prefix,iset,x1,qsc,abs(ipart(1)))
     &          -GetOnePDF(prefix,iset,x1,qsc,abs(ipart(1))+6)
          else
           xpdf1=GetOnePDF(prefix,iset,x1,qsc,ipart(1))
          end if
          if (ipart(2) < 0) then
           xpdf2=GetOnePDF(prefix,iset,x2,qsc,abs(ipart(2)))
     &          -GetOnePDF(prefix,iset,x2,qsc,abs(ipart(2))+6)
          else
           xpdf2=GetOnePDF(prefix,iset,x2,qsc,ipart(2))
          end if
          if (ipart(1) /= ipart(2)) then
           if (-ipart(2) < 0) then
            ypdf1=GetOnePDF(prefix,iset,x1,qsc,abs(-ipart(2)))
     &           -GetOnePDF(prefix,iset,x1,qsc,abs(-ipart(2))+6)
           else
            ypdf1=GetOnePDF(prefix,iset,x1,qsc,-ipart(2))
           end if
           if (-ipart(1) < 0) then
            ypdf2=GetOnePDF(prefix,iset,x2,qsc,abs(-ipart(1)))
     &           -GetOnePDF(prefix,iset,x2,qsc,abs(-ipart(1))+6)
           else
            ypdf2=GetOnePDF(prefix,iset,x2,qsc,-ipart(1))
           end if
          else
           ypdf1=0.d0
           ypdf2=0.d0
          end if
         else if (ipdf == 2) then
          xpdf1=Ctq6pdf(ipart(1),x1,qsc)
          xpdf2=Ctq6pdf(ipart(2),x2,qsc)
          if (ipart(1) /= ipart(2)) then
           ypdf1=Ctq6pdf(-ipart(2),x1,qsc)
           ypdf2=Ctq6pdf(-ipart(1),x2,qsc)
c
         else
           ypdf1=0.d0
           ypdf2=0.d0
          end if
         end if
        else if (ihad == 2) then     ! proton-proton scattering
         if (ipdf == 1) then
          if (ipart(1) < 0) then
           xpdf1=GetOnePDF(prefix,iset,x1,qsc,abs(ipart(1)))
     &          -GetOnePDF(prefix,iset,x1,qsc,abs(ipart(1))+6)
          else
           xpdf1=GetOnePDF(prefix,iset,x1,qsc,ipart(1))
          end if
          if (ipart(2) < 0) then
           xpdf2=GetOnePDF(prefix,iset,x2,qsc,abs(ipart(2)))
     &          -GetOnePDF(prefix,iset,x2,qsc,abs(ipart(2))+6)
          else
           xpdf2=GetOnePDF(prefix,iset,x2,qsc,ipart(2))
          end if
          if (ipart(1) /= ipart(2)) then
           if (ipart(2) < 0) then
            ypdf1=GetOnePDF(prefix,iset,x1,qsc,abs(ipart(2)))
     &           -GetOnePDF(prefix,iset,x1,qsc,abs(ipart(2))+6)
           else
            ypdf1=GetOnePDF(prefix,iset,x1,qsc,ipart(2))
           end if
           if (ipart(1) < 0) then
            ypdf2=GetOnePDF(prefix,iset,x2,qsc,abs(ipart(1)))
     &           -GetOnePDF(prefix,iset,x2,qsc,abs(ipart(1))+6)
           else
            ypdf2=GetOnePDF(prefix,iset,x2,qsc,ipart(1))
           end if
          else
           ypdf1=0.d0
           ypdf2=0.d0
          end if
         else if (ipdf == 2) then
          xpdf1=Ctq6pdf(ipart(1),x1,qsc)
          xpdf2=Ctq6pdf(ipart(2),x2,qsc)
          if (ipart(1) /= ipart(2)) then
           ypdf1=Ctq6pdf(ipart(2),x1,qsc)
           ypdf2=Ctq6pdf(ipart(1),x2,qsc)
          else
           ypdf1=0.d0
           ypdf2=0.d0
          end if
         end if
        end if
        crosskk=(xpdf1*xpdf2+ypdf1*ypdf2)*crosskk*dx1*dx2
        if (ipdf == 1) crosskk=crosskk/(x1*x2)
ckk_had<
       end if

       icall=icall+1

       if (imc == 1) then
        if (crosskk > crmax) then
         crmax=crosskk
         if (iscan == 0) imxup=1
        end if
       end if

       if (idis == 1 .and. iscan == 0) then
        if (iisr == 1 .or. ihad > 0) then
         if (iqprec == 1) then
          r_dp=r_qp
         end if
         call calcdis(r_dp,crosskk)
        else
         if (iqprec == 1) then
          p_dp=p
         end if
         call calcdis(p_dp,crosskk)
        end if
       end if

       if (imc == 1 .and. iscan == 0) then
        ieff=ieff+1
        write(11,*) crosskk
        do i=nisp+1,nepl
         if (iisr == 1 .or. ihad > 0) then
          write(11,'(4(e16.8))') r(:,i)
         else
          write(11,'(4(e16.8))') p(:,i)
         end if
        end do
       end if
      else            ! Introduced on 15 Nov. 2007
       crosskk=zero
       ntd0=ntd0+1
       return
      end if  !  dlinv

      end function crosskk
